from django.contrib import admin
from glinks.models import Blueprint, Glink, ImpressionTracking, ClickTracking

class GlinkAdmin(admin.ModelAdmin):
	fields = ['blueprint', 'name', 'weight', 'URL', 'image', 'impressions', 'clicks', 'mostClicksLocation', 'mostViewedLocation']

	def get_readonly_fields(self, request, obj=None):
		return ['impressions', 'clicks', 'mostClicksLocation', 'mostViewedLocation']

class ClickTrackingAdmin(admin.ModelAdmin):
	def get_readonly_fields(self, request, obj=None):
		return ['glink_id', 'latitude', 'longitude', 'country', 'city']


# Register your models here.
admin.site.register(Blueprint)
admin.site.register(Glink, GlinkAdmin)
admin.site.register(ImpressionTracking)
admin.site.register(ClickTracking, ClickTrackingAdmin)


from django.http import HttpResponseRedirect
from glinks.models import Glink, Blueprint, ClickTracking
from glinks.templatetags.utilities import *

# Create your views here.
def glink_counter(request, glink_id):
	glink = Glink.objects.get(pk=glink_id)
	glink.clicks = glink.clicks + 1
	glink.save()

	ip = getClientIpFromRequest(request)
	tracking_dict = getLocationFromIp(ip)

	clickTracking = ClickTracking(glink_id=glink, latitude=tracking_dict["Latitude"], 
				longitude=tracking_dict["Longitude"], country=tracking_dict["Country"], 
				city=tracking_dict["City"])
	clickTracking.save()

	return HttpResponseRedirect(glink.URL)


from django.db import models
from django.conf import settings

# Create your models here.
class Blueprint(models.Model):

	# Name of blueprint
	name   = models.CharField(max_length=50, null=True, blank=False)

	# Height/width in pixels
	height = models.IntegerField(null=True, blank=True)
	width  = models.IntegerField(null=True, blank=True)

	def __unicode__(self):
		return self.name

class Glink(models.Model):

	blueprint = models.ForeignKey(Blueprint, null=True)

	# Name (Issue #2 requested.)
	name = models.CharField(null=False, blank=False, max_length=200)

	# Glink image
	image  = models.ImageField(null=False, blank=False, upload_to="glink_images")

	# Glink target URL
	URL    = models.URLField(null=True, blank=True)

	weight = models.PositiveIntegerField(max_length=9, default=0)

	# Click/impression tracking
	clicks = models.BigIntegerField(null=False, blank=True, default=0)
	impressions = models.BigIntegerField(null=False, blank=True, default=0)

	def __unicode__(self):
		return self.name

	def mostClicksLocation(self):
		clickTracking = ClickTracking.objects.filter(glink_id=self)
		tracking_dict = {}

		for tracking in clickTracking:
			key = tracking.city + ", " + tracking.country
			tracking_dict[key] = 0

		for key in tracking_dict.keys():
			keys = key.split(",")
			tracking_at_locations = ClickTracking.objects.filter(city=keys[0], country=keys[1].strip())
			tracking_dict[key] = len(tracking_at_locations)

		sorted_tracking_dict = sorted(tracking_dict.items(), key=lambda x:x[1])

		top = ""
		if len(sorted_tracking_dict) >= 1:
			top = top + str(sorted_tracking_dict[-1][0]) + " clicked this ad " + str(sorted_tracking_dict[-1][1]) + " times."

		return top

	def mostViewedLocation(self):
		impressionTracking = ImpressionTracking.objects.filter(glink_id=self)
		tracking_dict = {}

		for tracking in impressionTracking:
			key = tracking.city + ", " + tracking.country
			tracking_dict[key] = 0

		for key in tracking_dict.keys():
			keys = key.split(",")
			tracking_at_locations = ImpressionTracking.objects.filter(city=keys[0], country=keys[1].strip())
			tracking_dict[key] = len(tracking_at_locations)

		sorted_tracking_dict = sorted(tracking_dict.items(), key=lambda x:x[1])

		top = ""
		if len(sorted_tracking_dict) >= 1:
			top = top + str(sorted_tracking_dict[-1][0]) + " viewed this ad " + str(sorted_tracking_dict[-1][1]) + " times."

		return top

	mostViewedLocation.short_description = "Most Impressions Location"
	mostViewedLocation.editable = False

	mostClicksLocation.short_description = "Most Clicks Location"
	mostClicksLocation.editable = False



class ImpressionTracking(models.Model):

	glink_id = models.ForeignKey(Glink, verbose_name="Parent Glink")

	latitude = models.CharField(null=False, blank=True, max_length=20)

	longitude = models.CharField(null=False, blank=True, max_length=20)

	country = models.CharField(null=False, blank=True, max_length=200)

	city = models.CharField(null=False, blank=True, max_length=200)

	def __unicode__(self):
		return "User from: " + str(self.country) + ", " + str(self.city)

class ClickTracking(models.Model):

	glink_id = models.ForeignKey(Glink, verbose_name="Parent Glink")

	latitude = models.CharField(null=False, blank=True, max_length=20)

	longitude = models.CharField(null=False, blank=True, max_length=20)

	country = models.CharField(null=False, blank=True, max_length=200)

	city = models.CharField(null=False, blank=True, max_length=200)

	def __unicode__(self):
		return "User from: " + str(self.country) + ", " + str(self.city)





from glinks.models import Glink, Blueprint, ImpressionTracking

import urllib


def possibleGlinkOptions():
	options = ["blueprint", "as"]
	return options

def glinkErrorLead():
	return "\nError in glinks:\n"

def generateWeightedListFromGlinks(glinks):
	weight_list = []
	for glink in glinks:
		if glink.weight not in weight_list:
			for x in range(glink.weight):
				weight_list.append(glink.weight)
	return weight_list

def getClientIpFromRequest(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip

def getLocationFromIp(ip):
	address = 'http://api.hostip.info/get_html.php?ip=' + ip + '&position=true'
	location_data = urllib.urlopen(address).read()
	data_list = [s.strip() for s in location_data.splitlines()]
	data_list.remove('')
	tracking_dict = {}
	for data in data_list:
		info_list = data.split(':')
		tracking_dict[info_list[0].strip()] = info_list[1].strip()
	return tracking_dict



from distutils.core import setup
import os

README = open(os.path.join(os.path.dirname(__file__), 'README.txt')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-glinks',
    version='0.2.1',
    author='Shawn Simon',
    author_email='shawn.simon.developer@gmail.com',
    packages=['glinks', "glinks.templatetags"],
    url='http://pypi.python.org/pypi/django-glinks/',
    license='LICENSE.txt (CC-BY-NC)',
    description='Interal add manager for django sites with ads not blocked by ad blocker.',
    long_description=open('README.txt').read(),
    install_requires=[
        "Django >= 1.6.0",
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ],
)